from base64 import b64encode

from django_digest.test.methods import WWWAuthenticateError, BaseAuth

class BasicAuth(BaseAuth):
    def authorization(self, request, response):
        if response is not None:
            challenges = self._authenticate_headers(response)
            if 'Basic' not in challenges:
                raise WWWAuthenticateError(
                    'Basic authentication unsupported for %s to %r.' %
                    (response.request['REQUEST_METHOD'],
                     response.request['PATH_INFO'])
                )
        return 'Basic %s' % b64encode(self.username + ':' +  self.password)



DEFAULT_REALM = 'DJANGO'

def get_setting(setting_name, default=None):
    from django.conf import settings

    if hasattr(settings, setting_name):
        return getattr(settings, setting_name)
    else:
        return default

def get_backend(setting_name, default_backend_class_path):
    path = get_setting(setting_name, default_backend_class_path)

    from django.core import exceptions
    from django.utils.importlib import import_module

    path_components = path.rsplit('.', 1)
    if not len(path_components) == 2:
        raise exceptions.ImproperlyConfigured('%s isn\'t a classname' % path)
    try:
        mod = import_module(path_components[0])
    except ImportError, e:
        raise exceptions.ImproperlyConfigured('Error importing module %s: "%s"' %
                                              (path_components[0], e))

    try:
        cls = getattr(mod, path_components[1])
    except AttributeError:
        raise exceptions.ImproperlyConfigured('module "%s" does not define a "%s" class' %
                                              (path_components[0], path_components[1]))

    return cls()



import md5

from django.contrib.auth.models import User
from django.db import models
from django.db.models.signals import post_save

from python_digest import calculate_partial_digest

from django_digest.utils import get_setting, DEFAULT_REALM

REALM = get_setting('DIGEST_REALM', DEFAULT_REALM)

class UserNonce(models.Model):
    user = models.ForeignKey(User)
    nonce = models.CharField(max_length=100, unique=True, db_index=True)
    count = models.IntegerField(null=True)
    last_used_at = models.DateTimeField(null=False)
    class Meta:
        ordering = ('last_used_at',)

class PartialDigest(models.Model):
    user = models.ForeignKey(User)
    partial_digest = models.CharField(max_length=100)
    
_postponed_partial_digests = {}

def _store_partial_digest(user):
    PartialDigest.objects.filter(user=user).delete()
    PartialDigest.objects.create(user=user,
                                 partial_digest=_postponed_partial_digests[user.password])

_old_set_password = User.set_password

def _new_set_password(user, password):
    _old_set_password(user, password)
    
    partial_digest = calculate_partial_digest(user.username, REALM, password)
    _postponed_partial_digests[user.password] = partial_digest
    
User.set_password = _new_set_password

def _persist_partial_digest(sender, instance=None, **kwargs):
    if instance is None:
        return
    if instance.password in _postponed_partial_digests:
        _store_partial_digest(instance)
        del _postponed_partial_digests[instance.password]

post_save.connect(_persist_partial_digest, sender=User)



import types

from functools import partial
from operator import isCallable

from django.http import HttpRequest, HttpResponse

from decorator import decorator

from django_digest import HttpDigestAuthenticator

def _httpdigest(authenticator, f, *args, **kwargs):
    # 'f' might be a function, in which case args[0] is 'request'
    if len(args) >= 1 and isinstance(args[0], HttpRequest):
        request = args[0]
    # 'f' could also be a method, in which case args[0] is 'self' and
    # args[1] is 'request'
    elif len(args) >= 2 and isinstance(args[1], HttpRequest):
        request = args[1]
    else:
        raise Exception("Neither args[0] nor args[1] is an HttpRequest.")

    if not authenticator.authenticate(request):
        return authenticator.build_challenge_response()

    response = f(*args, **kwargs)
    if hasattr(response, 'status_code') and response.status_code in [401, 403]:
        return authenticator.build_challenge_response()

    return response
    

def httpdigest(*args, **kwargs):
    '''
    May be used in one of three ways:
    * as a decorator factory (with the arguments being parameters to an instance of
    HttpDigestAuthenticator used to protect the decorated view)
    * as a decorator (protecting the decorated view with a default-constructed instance of
    HttpDigestAuthenticator)
    * as a decorator factory (with the argument being a pre-constructed HttpDigestAuthenticator
    instance used to protect the decorated view)
    '''
    if len(args) == 1 and not kwargs and isCallable(args[0]):
        authenticator = HttpDigestAuthenticator()
        return decorator(partial(_httpdigest, authenticator), args[0])

    if len(args) == 1 and not kwargs and isinstance(args[0], HttpDigestAuthenticator):
        authenticator = args[0]
    else:
        authenticator = HttpDigestAuthenticator(*args, **kwargs)

    return lambda(f): decorator(partial(_httpdigest, authenticator), f)



from datetime import datetime

from django.contrib.auth.models import User
from django.db import IntegrityError

from django_digest.models import UserNonce, PartialDigest
from django_digest.utils import get_setting

class AccountStorage(object):
    def get_partial_digest(self, username):
        try:
            return PartialDigest.objects.get(user__username=username).partial_digest
        except PartialDigest.DoesNotExist:
            return None

    def get_user(self, username):
        try:
            return User.objects.get(username=username)
        except User.DoesNotExist:
            return None

class NonceStorage(object):
    def _expire_nonces_for_user(self, user):
        try:
            delete_older_than = UserNonce.objects.filter(user=user).order_by(
                '-last_used_at')[31].last_used_at
        except IndexError:
            # There 30 or less nonces stored
            return

        UserNonce.objects.filter(user=user, last_used_at__lte=delete_older_than).delete()


    def update_existing_nonce(self, user, nonce, nonce_count):
        nonce_query_set = UserNonce.objects.filter(user=user,
                                                   nonce=nonce)
        if not nonce_count == None:
            nonce_query_set = nonce_query_set.filter(count__lt=nonce_count)
        
        # if no rows are updated, either the nonce isn't in the DB, it's for a different
        # user, or the count is bad
        return nonce_query_set.update(last_used_at=datetime.now(), count=nonce_count)

    def store_nonce(self, user, nonce, nonce_count):
        self._expire_nonces_for_user(user)

        try:
            UserNonce.objects.create(user=user, nonce=nonce, count=nonce_count,
                                     last_used_at=datetime.now())
        except IntegrityError:
            return False
        
        return True



from __future__ import with_statement
from django.test import TestCase

import time

from mocker import Mocker, expect

import python_digest
from python_digest.utils import parse_parts

from django.conf import settings
from django.contrib.auth.models import User
from django.http import HttpRequest

from django_digest import HttpDigestAuthenticator
from django_digest.backend.db import AccountStorage
from django_digest.decorators import httpdigest
from django_digest.middleware import HttpDigestMiddleware
from django_digest.models import PartialDigest
from django_digest.utils import get_setting, get_backend, DEFAULT_REALM

class DummyBackendClass(object):
    pass

class OtherDummyBackendClass(object):
    pass

class UtilsTest(TestCase):
    def test_get_setting(self):
        settings.A_PRESENT_SETTING = 'hello'
        settings.A_FALSE_SETTING = False
        self.assertEqual('hello', get_setting('A_PRESENT_SETTING', 'blah'))
        self.assertEqual('blah', get_setting('AN_ABSENT_SETTING', 'blah'))
        self.assertEqual(False, get_setting('A_FALSE_SETTING', 'blah'))

    def test_get_backend(self):
        settings.A_PRESENT_BACKEND_SETTING = 'django_digest.tests.DummyBackendClass'
        self.assertEqual(DummyBackendClass,
                         type(get_backend('A_PRESENT_BACKEND_SETTING',
                                          'django_digest.tests.OtherDummyBackendClass')))
        self.assertEqual(OtherDummyBackendClass,
                         type(get_backend('AN_ABSENT_BACKEND_SETTING',
                                          'django_digest.tests.OtherDummyBackendClass')))


class DjangoDigestTests(TestCase):
    def setUp(self):
        self.mocker = Mocker()
        self.nonce = python_digest.calculate_nonce(time.time(), secret=settings.SECRET_KEY)
        
    def create_mock_request(self, username='dummy-username', realm=None,
                            method='GET', uri='/dummy/uri', nonce=None, request_digest=None,
                            algorithm=None, opaque='dummy-opaque', qop='auth', nonce_count=1,
                            client_nonce=None, password='password', request_path=None):
        if not realm:
            realm = get_setting('DIGEST_REALM', DEFAULT_REALM)
        if not nonce:
            nonce=self.nonce
        if not request_path:
            request_path = uri
        header = python_digest.build_authorization_request(
            username=username, realm=realm, method=method, uri=uri, nonce=nonce, opaque=opaque,
            nonce_count=nonce_count, password=password, request_digest=request_digest,
            client_nonce=client_nonce)

        request = self.create_mock_request_for_header(header)

        expect(request.method).result(method)
        expect(request.path).result(request_path)

        return request

    def create_mock_request_for_header(self, header):
        request = self.mocker.mock(HttpRequest, count=False)

        # bug in mocker: https://bugs.launchpad.net/mocker/+bug/179072
        try:
            'HTTP_AUTHORIZATION' in request.META
        except:
            pass
        self.mocker.result(not header == None)

        if header:
            expect(request.META['HTTP_AUTHORIZATION']).count(0, None).result(header)

        return request

    def test_build_challenge_response(self):
        response = HttpDigestAuthenticator().build_challenge_response()
        self.assertEqual(401, response.status_code)
        self.assertEqual('digest ', response["WWW-Authenticate"][:7].lower())
        parts = parse_parts(response["WWW-Authenticate"][7:])
        self.assertEqual(get_setting('DIGEST_REALM', DEFAULT_REALM), parts['realm'])

    def test_decorator_authenticated_with_parens(self):
        response = self.mocker.mock(count=False)
        expect(response.status_code).result(200)

        @httpdigest()
        def test_view(request):
            return response

        testuser = User.objects.create_user('testuser', 'user@example.com', 'pass')
        request = self.create_mock_request(username=testuser.username, password='pass')
        request.user = testuser

        with self.mocker:
            self.assertEqual(response, test_view(request))
        
    def test_decorator_authenticated_without_parens(self):
        response = self.mocker.mock(count=False)
        expect(response.status_code).result(200)

        @httpdigest
        def test_view(request):
            return response

        testuser = User.objects.create_user('testuser', 'user@example.com', 'pass')
        request = self.create_mock_request(username=testuser.username, password='pass')
        request.user = testuser

        with self.mocker:
            self.assertEqual(response, test_view(request))
        
    def test_decorator_authenticated_with_full_uri(self):
        response = self.mocker.mock(count=False)
        expect(response.status_code).result(200)

        @httpdigest
        def test_view(request):
            return response

        testuser = User.objects.create_user('testuser', 'user@example.com', 'pass')
        request = self.create_mock_request(username=testuser.username, password='pass',
                                           uri='http://server:8000/some/path?q=v',
                                           request_path='/some/path')
        request.user = testuser

        with self.mocker:
            self.assertEqual(response, test_view(request))
        
    def test_decorator_with_realm_mismatch(self):
        response = self.mocker.mock(count=False)

        @httpdigest
        def test_view(request):
            return response

        testuser = User.objects.create_user('testuser', 'user@example.com', 'pass')
        request = self.create_mock_request(username=testuser.username, password='pass',
                                           realm='BAD_REALM')

        with self.mocker:
            final_response = test_view(request)
            self.assertEqual(401, final_response.status_code)
            self.assertTrue('WWW-Authenticate' in final_response)
        
    def test_decorator_unauthenticated_and_custom_settings(self):
        response = self.mocker.mock(count=False)
        expect(response.status_code).result(200)

        @httpdigest(realm='MY_TEST_REALM')
        def test_view(request):
            return response

        testuser = User.objects.create_user('testuser', 'user@example.com', 'pass')
        request = self.create_mock_request_for_header(None)
        request.user = testuser

        with self.mocker:
            final_response = test_view(request)
            self.assertEqual(401, final_response.status_code)
            self.assertTrue('WWW-Authenticate' in final_response)
            self.assertTrue('MY_TEST_REALM' in final_response['WWW-Authenticate'])
        
    def test_decorator_with_pre_constructed_authenticator(self):
        response = self.mocker.mock(count=False)
        expect(response.status_code).result(200)

        @httpdigest(HttpDigestAuthenticator(realm='MY_TEST_REALM'))
        def test_view(request):
            return response

        testuser = User.objects.create_user('testuser', 'user@example.com', 'pass')
        request = self.create_mock_request_for_header(None)
        request.user = testuser

        with self.mocker:
            final_response = test_view(request)
            self.assertEqual(401, final_response.status_code)
            self.assertTrue('WWW-Authenticate' in final_response)
            self.assertTrue('MY_TEST_REALM' in final_response['WWW-Authenticate'])

    def test_disable_nonce_count_enforcement(self):
        old_digest_enforce_nonce_count = None
        if hasattr(settings, 'DIGEST_ENFORCE_NONCE_COUNT'):
            old_digest_enforce_nonce_count = settings.DIGEST_ENFORCE_NONCE_COUNT
        settings.DIGEST_ENFORCE_NONCE_COUNT = False
        try:
            testuser = User.objects.create_user('testuser', 'user@example.com', 'pass')
            first_request = self.create_mock_request(username=testuser.username,
                                                     password='pass')
            first_request.user = testuser
            
            # same nonce, same nonce count, will fail
            second_request = self.create_mock_request(username=testuser.username,
                                                      password='pass')
            second_request.user = testuser
            with self.mocker:
                self.assertTrue(HttpDigestAuthenticator().authenticate(first_request))
                self.assertTrue(HttpDigestAuthenticator().authenticate(second_request))
        finally:
            if old_digest_enforce_nonce_count is not None:
                settings.DIGEST_ENFORCE_NONCE_COUNT = old_digest_enforce_nonce_count


    def test_authenticate(self):
        testuser = User.objects.create_user('testuser', 'user@example.com', 'pass')
        otheruser = User.objects.create_user('otheruser', 'otheruser@example.com', 'pass')

        first_request = self.create_mock_request(username=testuser.username,
                                           password='pass')
        first_request.user = testuser

        # same nonce, same nonce count, will fail
        second_request = self.create_mock_request(username=testuser.username,
                                                  password='pass')

        # same nonce, new nonce count, it works
        third_request = self.create_mock_request(username=testuser.username,
                                                 password='pass', nonce_count=2)
        third_request.user = testuser

        # an invalid request
        fourth_request = self.create_mock_request_for_header('Digest blah blah blah')

        # an invalid request
        fifth_request = self.create_mock_request_for_header(None)

        # an invalid nonce
        sixth_request = self.create_mock_request(
            username=testuser.username, password='pass', nonce_count=1,
            nonce=python_digest.calculate_nonce(time.time(), secret='bad secret'))

        # an invalid request digest (wrong password)
        seventh_request = self.create_mock_request(
            username=testuser.username, password='wrong', nonce_count=3)

        # attack attempts / failures don't invalidate the session or increment nonce_cont
        eighth_request = self.create_mock_request(username=testuser.username,
                                                  password='pass', nonce_count=3)
        eighth_request.user = testuser

        # mismatched URI
        ninth_request = self.create_mock_request(username=testuser.username,
                                                 password='pass', nonce_count=4,
                                                 request_path='/different/path')

        # stale nonce
        tenth_request = self.create_mock_request(
            username=testuser.username, password='pass', nonce_count=4,
            nonce=python_digest.calculate_nonce(time.time()-60000, secret=settings.SECRET_KEY))

        # once the nonce is used by one user, can't be reused by another
        eleventh_request = self.create_mock_request(username=otheruser.username,
                                                  password='pass', nonce_count=4)

        # if the partial digest is not in the DB, authentication fails
        twelfth_request = self.create_mock_request(username=testuser.username,
                                                   password='pass', nonce_count=3)
        
        # a request for Basic auth
        thirteenth_request = self.create_mock_request_for_header('Basic YmxhaDpibGFo')

        with self.mocker:
            self.assertTrue(HttpDigestAuthenticator.contains_digest_credentials(first_request))
            self.assertTrue(HttpDigestAuthenticator().authenticate(first_request))
            self.assertFalse(HttpDigestAuthenticator().authenticate(second_request))
            self.assertTrue(HttpDigestAuthenticator().authenticate(third_request))
            self.assertFalse(HttpDigestAuthenticator().authenticate(fourth_request))
            self.assertFalse(HttpDigestAuthenticator().authenticate(fifth_request))
            self.assertFalse(HttpDigestAuthenticator().authenticate(sixth_request))
            self.assertFalse(HttpDigestAuthenticator().authenticate(seventh_request))
            self.assertTrue(HttpDigestAuthenticator().authenticate(eighth_request))
            self.assertFalse(HttpDigestAuthenticator().authenticate(ninth_request))
            self.assertFalse(HttpDigestAuthenticator().authenticate(tenth_request))
            self.assertFalse(HttpDigestAuthenticator().authenticate(eleventh_request))

            PartialDigest.objects.all().delete()
            self.assertFalse(HttpDigestAuthenticator().authenticate(twelfth_request))
            self.assertFalse(HttpDigestAuthenticator().authenticate(thirteenth_request))

class ModelsTests(TestCase):
    def test_partial_digest_creation(self):
        user = User.objects.create(username='testuser', email='testuser@example.com')
        self.assertEqual(0,PartialDigest.objects.count())
        user.set_password('password')
        self.assertEqual(0,PartialDigest.objects.count())
        user.save()
        self.assertEqual(1,PartialDigest.objects.count())

class MiddlewareTests(TestCase):
    def setUp(self):
        self.mocker = Mocker()

    def test_valid_login(self):
        authenticator = self.mocker.mock()
        request = self.mocker.mock()
        expect(authenticator.authenticate(request)).result(True)
        with self.mocker:
            self.assertEqual(
                None, HttpDigestMiddleware(authenticator=authenticator).process_request(request))

    def test_no_login_and_not_required(self):
        authenticator = self.mocker.mock()
        request = self.mocker.mock()
        expect(authenticator.authenticate(request)).result(False)
        expect(authenticator.contains_digest_credentials(request)).result(False)
        with self.mocker:
            self.assertEqual(
                None, HttpDigestMiddleware(authenticator=authenticator).process_request(request))

    def test_no_login_and_required(self):
        authenticator = self.mocker.mock(count=False)
        request = self.mocker.mock()
        response = self.mocker.mock()
        expect(authenticator.authenticate(request)).result(False)
        expect(authenticator.contains_digest_credentials(request)).result(False)
        expect(authenticator.build_challenge_response()).result(response)
        with self.mocker:
            self.assertEqual(
                response,
                HttpDigestMiddleware(authenticator=authenticator,
                                     require_authentication=True).process_request(request))

    def test_process_response_401(self):
        authenticator = self.mocker.mock(count=False)
        request = self.mocker.mock()
        response = self.mocker.mock(count=False)
        challenge_response = self.mocker.mock()
        expect(response.status_code).result(401)
        expect(authenticator.build_challenge_response()).result(challenge_response)
        with self.mocker:
            self.assertEqual(
                challenge_response,
                HttpDigestMiddleware(authenticator=authenticator).process_response(
                    request, response))

    def test_process_response_403(self):
        authenticator = self.mocker.mock(count=False)
        request = self.mocker.mock()
        response = self.mocker.mock(count=False)
        challenge_response = self.mocker.mock()
        expect(response.status_code).result(403)
        expect(authenticator.build_challenge_response()).result(challenge_response)
        with self.mocker:
            self.assertEqual(
                challenge_response,
                HttpDigestMiddleware(authenticator=authenticator).process_response(
                    request, response))
        
    def test_process_response_200(self):
        authenticator = self.mocker.mock(count=False)
        request = self.mocker.mock()
        response = self.mocker.mock(count=False)
        expect(response.status_code).result(200)
        with self.mocker:
            self.assertEqual(
                response,
                HttpDigestMiddleware(authenticator=authenticator).process_response(
                    request, response))
        
    def test_process_response_404(self):
        authenticator = self.mocker.mock(count=False)
        request = self.mocker.mock()
        response = self.mocker.mock(count=False)
        expect(response.status_code).result(404)
        with self.mocker:
            self.assertEqual(
                response,
                HttpDigestMiddleware(authenticator=authenticator).process_response(
                    request, response))

class DbBackendTests(TestCase):
    def test_get_partial_digest(self):
        user1 = User.objects.create(username='user1', email='user1@example.com')
        user2 = User.objects.create_user(username='user2', email='user2@example.com',
                                         password='pass')
        self.assertEqual(None, AccountStorage().get_partial_digest(user1.username))
        self.assertTrue(AccountStorage().get_partial_digest(user2.username) is not None)
        self.assertEqual(None, AccountStorage().get_partial_digest('user3'))

    def test_get_user(self):
        user1 = User.objects.create(username='user1', email='user1@example.com')
        user2 = User.objects.create_user(username='user2', email='user2@example.com',
                                         password='pass')
        self.assertEqual(user1, AccountStorage().get_user(user1.username))
        self.assertEqual(user2, AccountStorage().get_user(user2.username))
        self.assertEqual(None, AccountStorage().get_user('user3'))



import django.test

from django_digest.test.methods.basic import BasicAuth
from django_digest.test.methods.detect import DetectAuth
from django_digest.test.methods.digest import DigestAuth

class Client(django.test.Client):
    AUTH_METHODS = {'Basic': BasicAuth,
                    'Digest': DigestAuth}

    def __init__(self, *args, **kwargs):
        super(Client, self).__init__(*args, **kwargs)
        self.clear_authorization()

    def request(self, **request):
        if self.auth_method:
            request.update(self.auth_method(request))
        response = super(Client, self).request(**request)
        if response.status_code == 401 and self.auth_method:
            # Try to authenticate
            request.update(self.auth_method(request, response))
            response = super(Client, self).request(**request)
        return response

    def set_authorization(self, username, password, method=None):
        self.username = username
        self.password = password
        if method is None:
            self.auth_method = DetectAuth(client=self,
                                          username=username,
                                          password=password)
        else:
            self.auth_method = self.AUTH_METHODS[method](username=username,
                                                         password=password)

    def clear_authorization(self):
        self.username = None
        self.password = None
        self.auth_method = None




from south.db import db
from django.db import models
from django_digest.models import *

class Migration:
    
    def forwards(self, orm):
        
        # Adding model 'UserNonce'
        db.create_table('django_digest_usernonce', (
            ('id', orm['django_digest.UserNonce:id']),
            ('user', orm['django_digest.UserNonce:user']),
            ('nonce', orm['django_digest.UserNonce:nonce']),
            ('count', orm['django_digest.UserNonce:count']),
            ('last_used_at', orm['django_digest.UserNonce:last_used_at']),
        ))
        db.send_create_signal('django_digest', ['UserNonce'])
        
        # Adding model 'PartialDigest'
        db.create_table('django_digest_partialdigest', (
            ('id', orm['django_digest.PartialDigest:id']),
            ('user', orm['django_digest.PartialDigest:user']),
            ('partial_digest', orm['django_digest.PartialDigest:partial_digest']),
        ))
        db.send_create_signal('django_digest', ['PartialDigest'])
        
    
    
    def backwards(self, orm):
        
        # Deleting model 'UserNonce'
        db.delete_table('django_digest_usernonce')
        
        # Deleting model 'PartialDigest'
        db.delete_table('django_digest_partialdigest')
        
    
    
    models = {
        'auth.group': {
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '80', 'unique': 'True'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'unique_together': "(('content_type', 'codename'),)"},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'max_length': '30', 'unique': 'True'})
        },
        'contenttypes.contenttype': {
            'Meta': {'unique_together': "(('app_label', 'model'),)", 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'django_digest.partialdigest': {
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'partial_digest': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        },
        'django_digest.usernonce': {
            'count': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_used_at': ('django.db.models.fields.DateTimeField', [], {}),
            'nonce': ('django.db.models.fields.CharField', [], {'max_length': '100', 'unique': 'True', 'db_index': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        }
    }
    
    complete_apps = ['django_digest']



 #!/usr/bin/env python
from setuptools import setup

setup(
    name='django-digest',
    version='1.5',
    description=('An implementation of HTTP Digest Authentication for Django.'),
    long_description=(
"""
django-digest supplies a middleware (HttpDigestMiddleware) that may installed to protect access
to all URLs, a decorator (@httpdigest) that may be applied to selected view functions, and a
simple class (HttpDigestAuthenticator) that can be used to implement custom authentication
scenarios.

django-digest also supplies a subclass of django.test.Client that is able to perform Digest and
Basic authentication.
"""
    ),
    author='Akoha Inc.',
    author_email='adminmail@akoha.com',
    url='http://bitbucket.org/akoha/django-digest/',
    packages=['django_digest',
              'django_digest.backend',
              'django_digest.migrations',
              'django_digest.test',
              'django_digest.test.methods'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules'
     ],
    zip_safe=False,
)



from django_digest.test.methods import BaseAuth, WWWAuthenticateError

class DetectAuth(BaseAuth):
    def __init__(self, client, *args, **kwargs):
        super(DetectAuth, self).__init__(*args, **kwargs)
        self.client = client
        self.selected_method = None

    def authorization(self, request, response):
        if self.selected_method is None:
            if response is None or response.status_code != 401:
                # we haven't previously authenticated, and we don't need to (or don't
                # know that we need to)
                return None
            
            challenges = self._authenticate_headers(response)
            for method, challenge in challenges.iteritems():
                # Loop through possible challenges to find the first one we
                # can handle.
                if method not in self.client.AUTH_METHODS:
                    continue
                self.selected_method = self.client.AUTH_METHODS[method](username=self.username,
                                                                        password=self.password)
                break

        if self.selected_method is None:
            raise WWWAuthenticateError(
                '%r authentication methods unsupported for %s to %r.' %
                (tuple(self.client.AUTH_METHODS),
                 response.request['REQUEST_METHOD'], response.request['PATH_INFO'])
                )

        return self.selected_method.authorization(request, response)



from urllib import quote

from python_digest import build_authorization_request
from django_digest.test.methods import BaseAuth, WWWAuthenticateError


class DigestAuth(BaseAuth):
    def __init__(self, *args, **kwargs):
        super(DigestAuth, self).__init__(*args, **kwargs)
        self.nonce_count = 0
        self.digest_challenge = None

    def authorization(self, request, response):
        if response is not None:
            challenges = self._authenticate_headers(response)
            if 'Digest' not in challenges:
                raise WWWAuthenticateError(
                    'Digest authentication unsupported for %s to %r.' %
                    (response.request['REQUEST_METHOD'],
                     response.request['PATH_INFO'])
                )
            self.digest_challenge = challenges['Digest']
        elif self.digest_challenge is None:
            return
        self.nonce_count += 1
        return build_authorization_request(
            username=self.username,
            method=request['REQUEST_METHOD'],
            uri=quote(request['PATH_INFO']),
            nonce_count=self.nonce_count,
            digest_challenge=self.digest_challenge,
            password=self.password
        )



from django.utils.datastructures import SortedDict


class WWWAuthenticateError(Exception):
    pass


class BaseAuth(object):
    def __init__(self, username, password):
        self.username = username
        self.password = password

    def _authenticate_headers(self, response):
        return SortedDict([(value.split(' ', 1)[0], value)
                           for header, value in response.items()
                           if header == 'WWW-Authenticate'])

    def _update_headers(self, request, response):
        auth = self.authorization(request, response)
        return auth and {'HTTP_AUTHORIZATION': auth} or {}

    def __call__(self, request, response=None):
        return self._update_headers(request, response)
